@@ -14,21 +14,23 @@
 // along with this program.  If not, see <http://www.gnu.org/licenses/>
 //
 
-println("ketcindymv(20180615) loaded");
+println("ketcindymv(20181224) loaded");
 
 //help:start();
 
+////%Setpara start////
 Setpara(path,fstr,sL):=Setpara(path,fstr,sL,[],[]); 
 Setpara(pathorg,fstr,sL,options):= 
     Setpara(pathorg,fstr,sL,options,[]);
 Setpara(pathorg,fstr,sL,options,optionsanim):=(
 //help:Setpara("folder name","funstr","range",options,optionsanim);
 //help:Setpara(folder,funstr,range,options);
-//help:Setpara(options=["m/r","Div=25"]);
+//help:Setpara(options=["Div=25"]);
 //help:Setpara(options2anim1=["Frate=10","Scale=1"]);
 //help:Setpara(options2anim2=["OpA=[loop,controls,buttonsize=3mm]"]);
 //help:Setpara(options2anim3=["OpA=+step"]);
-//help:Setpara(options2anim4=["Mag=1600","Title=folder"]);
+//help :Setpara(options2anim4=["Mag=1600","Title=folder"]);
+//help:Setpara(options2anim4=["Title=folder"]);
   regional(path,tmp,tmp1,tmp2,tmp3,eqL,ndiv,gtmp,ctmp,nn,flg);
   if(length(pathorg)==0, path=Slidename,path=pathorg);//17.04.10
   ParaPath=path;
@@ -54,6 +56,10 @@ Setpara(pathorg,fstr,sL,options,optionsanim):=(
     ParaOp=append(ParaOp,"m");//180604
   );
   ParaOpAnim=optionsanim;
+ if(ADDAXES=="1", //181224rom
+    Drwxy();
+//    ADDAXES="0";
+  ); //181224to
   GLISTback=GLIST;
   GCLISTback=GCLIST;
   GOUTLISTback=GOUTLIST;
@@ -66,15 +72,12 @@ Setpara(pathorg,fstr,sL,options,optionsanim):=(
   COM2ndlistback=COM2ndlist;
   Dirworkbkup=Dirwork; //0611from
   Dirworksubbkup=Dirwork+pathsep()+ParaPath;
-  Changework(Dirworksubbkup);
-  Changework(Dirworkbkup);
   Fheadbkup=Fhead; //0611to
   gtmp=GLISTback;
   tmp1=select(1..(length(gtmp)),indexof(gtmp_#,"ReadOutData")>0); //180614from
   tmp2=[];tmp3=[];
   if(length(tmp1)>0, //180615
     nn=tmp1_(length(tmp1));
-//  forall(tmp1,nn,
 	tmp2=append(tmp2,nn);
     flg=0;
     tmp=min([nn+40,length(gtmp)]);
@@ -100,7 +103,9 @@ Setpara(pathorg,fstr,sL,options,optionsanim):=(
   );
   COM2ndlistback=ctmp;
 );
+////%Setpara end////
 
+////%Parafolder start////
 Parafolder():=Parafolder(ParaFstr,ParaSL,ParaOp);
 Parafolder(fstr,sL):=Parafolder(fstr,sL,[]);
 Parafolder(fstr,sL,optionorg):=(
@@ -111,10 +116,9 @@ Parafolder(fstr,sL,optionorg):=(
 //help:Parafolder(optionsadd=[""Outfile=n",Pause=10(ms)"]); 
 //help:Paraslide(para=folder:layery:pos:input,scale); 
 //help:Paraslide(para=folder:layery:pos:include:[width=100]); 
-  regional(nn,tmp,tmp1,tmp2,strL,eqL,waiting,outflg,pause,
-        mkr,mktex,options,sfL,dirbkup,fbkup,pfile,ctr,flg,varL);
-  Changework(Dirworkbkup);
-  Setfiles(Fheadbkup);
+  regional(store,nn,tmp,tmp1,tmp2,strL,eqL,waiting,outflg,pause,
+        mkr,mktex,options,sfL,dirbkup,pfile,ctr,flg,varL,waitingR,timeC,timeR);
+  store=Fillblack(); //181125
   tmp=indexof(fstr,"(");
   tmp1=substring(fstr,tmp,length(fstr));
   tmp=indexof(tmp1,")");
@@ -127,7 +131,7 @@ Parafolder(fstr,sL,optionorg):=(
   if(length(strL)==0,options=append(options,"m"));
   mkr="A";
   mktex="A";
-  waiting=20;
+  waiting=30;
   outflg=0;
   pause=10;
   forall(eqL,
@@ -153,10 +157,12 @@ Parafolder(fstr,sL,optionorg):=(
       mkr="Y"; mktex="Y";
     );
   );
+  waitingR=min([length(sL)*waiting,120]);//180617
   Changework(Dirworksubbkup);
   SCEOUTPUT = openfile("all.r");
   println(SCEOUTPUT,"");
   closefile(SCEOUTPUT);//180608to
+  resetclock();//180617
   if(outflg==1, //180606from
     CommonMake=1;//180609
     ctr=1;
@@ -164,21 +170,17 @@ Parafolder(fstr,sL,optionorg):=(
       tmp="000000"+text(ctr);
       pfile="p"+substring(tmp,length(tmp)-3,length(tmp));
       Setfiles(pfile);
-      Changework(Dirworksubbkup);
       Movieframe(#);
-      Changework(Dirworksubbkup);//180608
       ctr=ctr+1;
     );
     CommonMake=-1;//180609
-    Setfiles(fbkup);
+    Setfiles(Fheadbkup);//180617
+    timeC=seconds();//180617
   ); //180606to
   wait(pause); //180610
-//  Changework(Dirworkbkup);
   sfL=[];
   ctr=1;
   forall(1..(length(sL)),nn,
-    Changework(Dirworkbkup);//180611(2lines)
-    Setfiles(Fheadbkup);
     GLIST=GLISTback;
     GCLIST=GCLISTback;
     GOUTLIST=GOUTLISTback;
@@ -191,18 +193,17 @@ Parafolder(fstr,sL,optionorg):=(
     COM2ndlist=COM2ndlistback;
     tmp="000000"+text(nn);
     pfile="p"+substring(tmp,length(tmp)-3,length(tmp));
-    Setfiles(pfile);
     Changework(Dirworksubbkup);
+    Setfiles(pfile);
     sfL=append(sfL,FnameR);
     if(outflg==1,
-      Changework(Dirworksubbkup);//180606
-      Setfiles(pfile); //180610(2lines)
       wait(pause);
       Movieframe(sL_nn);
       Changework(Dirworksubbkup);//180606
       ctr=ctr+1;
     ,
       Movieframe(sL_nn);
+      timeC=seconds();//180617
     );
     CommonMake=0;//180609
     Setfiles(pfile); //180608
@@ -213,8 +214,7 @@ Parafolder(fstr,sL,optionorg):=(
     );
   );
   wait(pause); //180610
-  Setfiles(fkbkup);
-  Changework(Dirworksubbkup);
+  Setfiles(Fheadbkup);
   GLIST=select(GLIST,indexof(#,"Projpara")==0);
   tmp1=replace(Dirworksubbkup,"\\","/"); //180611
   tmp1=replace(tmp1,"\","/");
@@ -250,7 +250,7 @@ Parafolder(fstr,sL,optionorg):=(
   if(ErrFlag!=-1,
     waiting=min([waiting,8])*length(ParaSL); //180610
     if(!isexists(Dirworksubbkup,Fnametex) % mktex=="Y",
-      CalcbyR("",cmdL,["Cat=n","m","Wait="+text(waiting)]);
+      CalcbyR("",cmdL,["Cat=n","m","Wait="+text(waitingR)]);
     ,
       println("Parafolder finished");
     );
@@ -258,10 +258,19 @@ Parafolder(fstr,sL,optionorg):=(
       parse(after);
     );
   );
+  if(ErrFlag!=-1,//180617from
+    timeR=seconds()-timeC;
+    println("C and R finished");
+    println("    C : "+Sprintf(timeC,2)+" sec");
+    println("    R : "+Sprintf(timeR,2)+" sec");
+  ); //180617to
   Changework(Dirworkbkup);
   Setfiles(Fheadbkup);
+  Fillrestore(store); //181125
 );
+////%Parafolder end////
 
+////%Animatefile start////
 Animatefile():=Animatefile(Dirwork,ParaPath);
 Animatefile(path,folder):=(
 //help:Animatefile();
@@ -274,16 +283,16 @@ Animatefile(path,folder):=(
   OpA="loop,controls,buttonsize=3mm";
   remflg=0;
   forall(eqL,
-    tmp1=Toupper(substring(#,0,2));
+    tmp1=Toupper(substring(#,0,1)); //181111
     tmp=indexof(#,"=");
     tmp2=substring(#,tmp,length(#));
-    if(tmp1=="FR",
+    if(tmp1=="F",
       FRate=tmp2;
     );
-    if(tmp1=="SC",
+    if(tmp1=="S",
       Scale=tmp2;
     );
-    if(tmp1=="OP",  // 17.12.07from
+    if(tmp1=="O",  // 17.12.07from
       if(length(tmp2)>0,
 	    tmp2=replace(tmp2,"[","");
         tmp2=replace(tmp2,"]","");
@@ -304,6 +313,7 @@ Animatefile(path,folder):=(
   tmp1=fileslist(tmp);
   tmp1=tokenize(tmp1,",");
   tmp1=select(tmp1,indexof(#,".tex")>0);
+  tmp1=sort(tmp1); //180617
   texfiles=[];//180609from
   forall(tmp1,
     tmp=indexof(#,".");
@@ -334,7 +344,9 @@ Animatefile(path,folder):=(
   closefile(SCEOUTPUT);
   println(fname+" has been generated");
 );
+////%Animatefile end////
 
+////%Mkanimation start////
 Mkanimation():=(
   regional(remflg,tmp,eqL,tmp1,tmp2);
   if(!isexists(Dirwork,ParaPath), //17.10.14from
@@ -347,8 +359,9 @@ Mkanimation(folder):=Mkanimation(Dirwork,folder);
 Mkanimation(path,folder):=(
 //help:Mkanimation();
 //help:Mkanimation(path,folder);
-  regional(Fheadbkup,Pathpdfbkup,tex,article,parent,
+  regional(store,Fheadbkup,Pathpdfbkup,tex,article,parent,
      eqL,mag,title,tmp,tmp1,tmp2,tmp3,flg);
+  store=Fillblack(); //181125
   tmp=Divoptions(ParaOpAnim); //17.11.25from
   eqL=tmp_5;
   mag="1600";
@@ -480,8 +493,11 @@ Mkanimation(path,folder):=(
   );
   Fhead=Fheadbkup;
   Pathpdf= Pathpdfbkup;
+  Fillrestore(store); //181125
 );
+////%Mkanimation end////
 
+////%Mkflipanime start////
 Mkflipanime():=(
   regional(remflg,tmp,eqL,tmp1,tmp2);
   if(!isexists(Dirwork,ParaPath), //17.10.14from
@@ -493,8 +509,9 @@ Mkflipanime(folder):=Mkflipanime(Dirwork,folder);
 Mkflipanime(path,folder):=(
 //help:Mkfilpanime();
 //help:Mkflipanime(path,folder);
-  regional(Fheadbkup,tex,article,parent,eqL,mag,title,
+  regional(store,Fheadbkup,tex,article,parent,eqL,mag,title,
      tmp,tmp1,tmp2,tmp3,texfiles,flg);
+  store=Fillblack(); //181125
   tmp=Divoptions(ParaOpAnim); //17.11.25from
   eqL=tmp_5;
   mag="1600";
@@ -580,6 +597,7 @@ Mkflipanime(path,folder):=(
   tmp=fileslist(Dirwork+pathsep()+folder);
   tmp=tokenize(tmp,",");
   tmp1=select(tmp,indexof(#,".tex")>0); //180609from
+  tmp1=sort(tmp1);//180617
   texfiles=[];
   forall(tmp1,
     tmp=indexof(#,".");
@@ -634,7 +652,9 @@ Mkflipanime(path,folder):=(
     kc();
   );
   Fhead=Fheadbkup;
+  Fillrestore(store); //181125
 );
+////%Mkflipanime end////
 
 //help:end();
 
