@@ -1,11 +1,22 @@
-if not modules then modules = { } end modules ['letterspace'] = {
-    version   = "2.7",
-    comment   = "companion to luaotfload-main.lua",
-    author    = "Hans Hagen, PRAGMA-ADE, Hasselt NL; adapted by Philipp Gesang",
-    copyright = "PRAGMA ADE / ConTeXt Development Team",
-    license   = "see context related readme files"
+-----------------------------------------------------------------------
+--         FILE:  luaotfload-letterspace.lua
+--  DESCRIPTION:  part of luaotfload / letterspacing
+-----------------------------------------------------------------------
+
+local ProvidesLuaModule = { 
+    name          = "luaotfload-letterspace",
+    version       = "2.95",       --TAGVERSION
+    date          = "2019-01-28", --TAGDATE
+    description   = "luaotfload submodule / color",
+    license       = "GPL v2.0",
+    copyright     = "PRAGMA ADE / ConTeXt Development Team",
+    author        = "Hans Hagen, PRAGMA-ADE, Hasselt NL; adapted by Philipp Gesang, Ulrike Fischer"
 }
 
+if luatexbase and luatexbase.provides_module then
+  luatexbase.provides_module (ProvidesLuaModule)
+end  
+
 --- This code diverged quite a bit from its origin in Context. Please
 --- do *not* report bugs on the Context list.
 
@@ -61,12 +72,16 @@ local todirect           = nodedirect.tonut
 local tonode             = nodedirect.tonode
 
 local insert_node_before = nodedirect.insert_before
-local free_node          = nodedirect.free
+local free_node          = nodedirect.free -- may cause double free
+local free_node          = function (n)
+  logreport ("term", 5, "letterspace", "not calling free_node(%d)", n)
+  -- free_node (n)
+end
 local copy_node          = nodedirect.copy
 local new_node           = nodedirect.new
 
 local nodepool           = nodedirect.pool
-local new_kern           = nodepool.kern
+-- local new_kern           = nodepool.kern -- UF removed 2017-07-14
 
 local nodecodes          = nodes.nodecodes
 
@@ -232,16 +247,21 @@ end
 ---                 character kerning functionality
 ---=================================================================---
 
+-- UF changed 2017-07-14
+local nodedirectnew = node.direct.new
+
 local kern_injector = function (fillup, kern)
-  if fillup then
-    local g = new_glue(kern)
-    local s = getfield(g, "spec")
-    setfield(s, "stretch", kern)
-    setfield(s, "stretch_order", 1)
-    return g
-  end
-  return new_kern(kern)
+ if fillup then
+   local g = nodedirectnew("glue")
+   setfield(g, "stretch", kern)
+   setfield(g, "stretch_order", 1)
+   return g
+ end
+   local g = nodedirectnew("kern")
+   setfield(g,"kern",kern)
+   return g
 end
+-- /UF
 
 local kernable_skip = function (n)
   local st = getsubtype (n)
@@ -348,7 +368,7 @@ kerncharacters = function (head)
             end
             start = c
             setfield(s, "components", nil)
-            --free_node(s) --> double free with multipart components
+            free_node(s) --> double free with multipart components
             c = getfield (start, "components")
           end
         end
