@@ -7,8 +7,8 @@
 
 local ProvidesLuaModule = { 
     name          = "luaotfload-parsers",
-    version       = "2.93",       --TAGVERSION
-    date          = "2018-10-28", --TAGDATE
+    version       = "2.95",       --TAGVERSION
+    date          = "2019-01-28", --TAGDATE
     description   = "luaotfload submodule / filelist",
     license       = "GPL v2.0"
 }
@@ -495,17 +495,6 @@ local splitcomma        = Ct((C(noncomma^1) + comma)^1)
 --doc]]------------------------------------------------------------------------
 
 
-local handle_normal_option = function (key, val)
-  val = stringlower(val)
-  --- the former “toboolean()” handler
-  if val == "true"  then
-    val = true
-  elseif val == "false" then
-    val = false
-  end
-  return key, val
-end
-
 --[[doc--
 
     Xetex style indexing begins at zero which we just increment before
@@ -513,35 +502,8 @@ end
 
 --doc]]--
 
-local handle_xetex_option = function (key, val)
-  val = stringlower(val)
-  local numeric = tonumber(val) --- decimal only; keeps colors intact
-  if numeric then --- ugh
-    if  mathceil(numeric) == numeric then -- integer, possible index
-      val = tostring(numeric + 1)
-    end
-  elseif val == "true"  then
-    val = true
-  elseif val == "false" then
-    val = false
-  end
-  return key, val
-end
-
---[[doc--
-
-    Instead of silently ignoring invalid options we emit a warning to
-    the log.
-
-    Note that we have to return a pair to please rawset(). This creates
-    an entry on the resulting features hash which will later be removed
-    during set_default_features().
-
---doc]]--
-
-local handle_invalid_option = function (opt)
-  logreport("log", 0, "load", "font option %q unknown.", opt)
-  return "", false
+local handle_xetex_option = function (val)
+  return tostring(1 + tonumber(val))
 end
 
 --[[doc--
@@ -646,19 +608,19 @@ local subfont           = P"(" * Cg((1 - S"()")^1, "sub") * P")"
 --- lookups -----------------------------------------------------------
 local fontname          = C((1-S":(/")^1)  --- like luatex-fonts
 local unsupported       = Cmt((1-S":(")^1, check_garbage)
-local combo             = P"combo:" * ws * Cg(combolist, "combo")
-local prefixed          = P"name:" * ws * Cg(fontname, "name")
+local combo             = Cg(P"combo", "lookup") * colon * ws
+                          * Cg(combolist, "name")
 --- initially we intended file: to emulate the behavior of
 --- luatex-fonts, i.e. no paths allowed. after all, we do have XeTeX
 --- emulation with the path lookup and it interferes with db lookups.
 --- turns out fontspec and other widely used packages rely on file:
 --- with paths already, so we’ll add a less strict rule here.  anyways,
 --- we’ll emit a warning.
-                        + P"file:" * ws * Cg(unsupported, "path")
-                        + P"file:" * ws * Cg(fontname, "file")
-                        + P"kpse:" * ws * Cg(fontname, "kpse")
-                        + P"my:" * ws * Cg(fontname, "my")
-local unprefixed        = Cg(fontname, "anon")
+local prefixed          = P"file:" * ws * Cg(Cc"path", "lookup")
+                          * Cg(unsupported, "name")
+                        + Cg(P"name" + "file" + "kpse" + "my", "lookup")
+                          * colon * ws * Cg(fontname, "name")
+local unprefixed        = Cg(Cc"anon", "lookup") * Cg(fontname, "name")
 --- Bracketed “path” lookups: These may contain any character except
 --- for unbalanced brackets. A backslash escapes any following
 --- character. Everything inside the outermost brackets is treated as
@@ -670,26 +632,25 @@ local path_escape       = backslash / "" * patterns.utf8char
 local path_content      = path_escape + (1 - brackets)
 local path_balanced     = { (path_content + V(2))^1
                           , lbrk * V(1)^-1 * rbrk }
-local path_lookup       = lbrk * Cg(Cs(path_balanced), "path") * rbrk
-                        * subfont^-1
+local path_lookup       = Cg(Cc"path", "lookup")
+                          * lbrk * Cg(Cs(path_balanced), "name") * rbrk
+                          * subfont^-1
 
 --- features ----------------------------------------------------------
-local field_char        = anum + S"+-.!?" --- sic!
-local field             = field_char^1
+local balanced_braces   = P{((1 - S'{}') + '{' * V(1) * '}')^0}
+local field_char        = '{' *   balanced_braces  * '}' + (1 - S'={}' - featuresep)
+local field             = '{' * C(balanced_braces) * '}' + C(field_char^1)
 --- assignments are “lhs=rhs”
 ---              or “+lhs=rhs” (Xetex-style)
 --- switches    are “+key” | “-key”
-local normal_option     = C(field) * ws * equals * ws * C(field) * ws
-local xetex_option      = P"+" * ws * normal_option
-local ignore_option     = (1 - equals - featuresep)^1
-                        * equals
-                        * (1 - featuresep)^1
-local assignment        = xetex_option  / handle_xetex_option
-                        + normal_option / handle_normal_option
-                        + ignore_option / handle_invalid_option
-local switch            = P"+" * ws * C(field) * Cc(true)
-                        + P"-" * ws * C(field) * Cc(false)
-                        +             C(field) * Cc(true)   --- default
+local normal_option     = field * ws * equals * ws * field
+local xetex_option      = P"+" * ws * (  field * ws * equals * ws
+                                               * (digit^1/handle_xetex_option)
+                                       + normal_option)
+local assignment        = xetex_option
+                        + normal_option
+local switch            = P"-"    * ws * field * Cc(false)
+                        + P"+"^-1 * ws * field * Cc(true)
 local feature_expr      = ws * Cg(assignment + switch) * ws
 local option            = feature_expr
 local feature_list      = Cf(Ct""
@@ -702,7 +663,7 @@ local feature_list      = Cf(Ct""
 local features          = Cg(feature_list, "features")
 local specification     = (prefixed + unprefixed)
                         * subfont^-1
-                        * modifier_list^-1
+                        * modifier_list
 local font_request      = Ct(path_lookup   * (colon^-1 * features)^-1
                            + combo --> TODO: feature list needed?
                            + specification * (colon    * features)^-1)
