@@ -1,17 +1,12 @@
-# $Id: TLPaper.pm 38608 2015-10-10 01:59:39Z preining $
+# $Id: TLPaper.pm 45795 2017-11-14 08:49:25Z preining $
 # TeXLive::TLPaper.pm - query/modify paper sizes for our various programs
-# Copyright 2008-2014 Norbert Preining
+# Copyright 2008-2017 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
-#
-# TODO TODO TODO
-# if no paperconfig line is found in paper_do_simple it adds to the end of the
-# file the new line, but this does not work on context because it 
-# has an \endinput at the end ... needs to be fixed
 
 package TeXLive::TLPaper;
 
-my $svnrev = '$Revision: 38608 $';
+my $svnrev = '$Revision: 45795 $';
 my $_modulerevision;
 if ($svnrev =~ m/: ([0-9]+) /) {
   $_modulerevision = $1;
@@ -33,11 +28,13 @@ BEGIN {
   @EXPORT = @EXPORT_OK;
 }
 
+my $prg = ($::prg ? $::prg : TeXLive::TLUtils::basename($0));
+
 =pod
 
 =head1 NAME
 
-C<TeXLive::TLPaper> -- paper size setting
+C<TeXLive::TLPaper> -- TeX Live paper size setting module
 
 =head1 SYNOPSIS
 
@@ -103,55 +100,81 @@ our %paper_config_name;
 
 
 my %xdvi_papersize = (
-  "a1"       => "59.4x84.0cm",
-  "a1r"      => "84.0x59.4cm",
-  "a2"       => "42.0x59.4cm",
-  "a2r"      => "59.4x42.0cm",
-  "a3"       => "29.7x42.0cm",
-  "a3r"      => "42.0x29.7cm",
-  "a4"       => "21.0x29.7cm",
-  "a4r"      => "29.7x21.0cm",
-  "a5"       => "14.85x21.0cm",
-  "a5r"      => "21.0x14.85cm",
-  "a6"       => "10.5x14.85cm",
-  "a6r"      => "14.85x10.5cm",
-  "a7"       => "7.42x10.5cm",
-  "a7r"      => "10.5x7.42cm",
-  "b1"       => "70.6x100.0cm",
-  "b1r"      => "100.0x70.6cm",
-  "b2"       => "50.0x70.6cm",
-  "b2r"      => "70.6x50.0cm",
-  "b3"       => "35.3x50.0cm",
-  "b3r"      => "50.0x35.3cm",
-  "b4"       => "25.0x35.3cm",
-  "b4r"      => "35.3x25.0cm",
-  "b5"       => "17.6x25.0cm",
-  "b5r"      => "25.0x17.6cm",
-  "b6"       => "13.5x17.6cm",
-  "b6r"      => "17.6x13.5cm",
-  "b7"       => "8.8x13.5cm",
-  "b7r"      => "13.5x8.8cm",
-  "c1"       => "64.8x91.6cm",
-  "c1r"      => "91.6x64.8cm",
-  "c2"       => "45.8x64.8cm",
-  "c2r"      => "64.8x45.8cm",
-  "c3"       => "32.4x45.8cm",
-  "c3r"      => "45.8x32.4cm",
-  "c4"       => "22.9x32.4cm",
-  "c4r"      => "32.4x22.9cm",
-  "c5"       => "16.2x22.9cm",
-  "c5r"      => "22.9x16.2cm",
-  "c6"       => "11.46x16.2cm",
-  "c6r"      => "16.2x11.46cm",
-  "c7"       => "8.1x11.46cm",
-  "c7r"      => "11.46x8.1cm",
-  "foolscap" => "13.5x17.0",
-  "ledger"   => "17.0x11.0",
-  "legal"    => "8.5x14",
-  "letter"   => "8.5x11",
-  "tabloid"  => "11.0x17.0",
-  "us"       => "8.5x11",
-  "usr"      => "11x8.5",
+  a0       => '841x1189mm',
+  a1       => '594x841mm',
+  a2       => '420x594mm',
+  a3       => '297x420mm',
+  a4       => '210x297mm',
+  a5       => '148x210mm',
+  a6       => '105x148mm',
+  a7       => '74x105mm',
+  a8       => '52x74mm',
+  a9       => '37x52mm',
+  a10      => '26x37mm',
+  a0r      => '1189x841mm',
+  a1r      => '841x594mm',
+  a2r      => '594x420mm',
+  a3r      => '420x297mm',
+  a4r      => '297x210mm',
+  a5r      => '210x148mm',
+  a6r      => '148x105mm',
+  a7r      => '105x74mm',
+  a8r      => '74x52mm',
+  a9r      => '52x37mm',
+  a10r     => '37x26mm',
+  b0       => '1000x1414mm',
+  b1       => '707x1000mm',
+  b2       => '500x707mm',
+  b3       => '353x500mm',
+  b4       => '250x353mm',
+  b5       => '176x250mm',
+  b6       => '125x176mm',
+  b7       => '88x125mm',
+  b8       => '62x88mm',
+  b9       => '44x62mm',
+  b10      => '31x44mm',
+  b0r      => '1414x1000mm',
+  b1r      => '1000x707mm',
+  b2r      => '707x500mm',
+  b3r      => '500x353mm',
+  b4r      => '353x250mm',
+  b5r      => '250x176mm',
+  b6r      => '176x125mm',
+  b7r      => '125x88mm',
+  b8r      => '88x62mm',
+  b9r      => '62x44mm',
+  b10r     => '44x31mm',
+  c0       => '917x1297mm',
+  c1       => '648x917mm',
+  c2       => '458x648mm',
+  c3       => '324x458mm',
+  c4       => '229x324mm',
+  c5       => '162x229mm',
+  c6       => '114x162mm',
+  c7       => '81x114mm',
+  c8       => '57x81mm',
+  c9       => '40x57mm',
+  c10      => '28x40mm',
+  c0r      => '1297x917mm',
+  c1r      => '917x648mm',
+  c2r      => '648x458mm',
+  c3r      => '458x324mm',
+  c4r      => '324x229mm',
+  c5r      => '229x162mm',
+  c6r      => '162x114mm',
+  c7r      => '114x81mm',
+  c8r      => '81x57mm',
+  c9r      => '57x40mm',
+  c10r     => '40x28mm',
+  us       => '8.5x11',
+  letter   => '8.5x11',
+  ledger   => '17x11',
+  tabloid  => '11x17',
+  usr      => '11x8.5',
+  legal    => '8.5x14',
+  legalr   => '14x8.5',
+  foolscap => '13.5x17.0',
+  foolscapr => '17.0x13.5',
 );
 
 my %pdftex_papersize = (
@@ -211,7 +234,7 @@ sub do_paper {
     my $sub = $paper{$prog}{'sub'};
     return(&$sub($texmfsysconfig, @args));
   } else {
-    tlwarn("$0: unknown paper program $prog ($texmfsysconfig,@args)\n");
+    tlwarn("$prg: unknown paper program $prog ($texmfsysconfig,@args)\n");
     return($F_ERROR);
   }
   return ($F_OK); # not reached
@@ -253,7 +276,7 @@ sub find_paper_file {
     }
   }
 
-  debug("$0: found no paper file for $progname (from $cmd)\n") if ! $ret;
+  debug("$prg: found no paper file for $progname (from $cmd)\n") if ! $ret;
   return $ret;
 }
 
@@ -299,13 +322,11 @@ sub paper_xdvi {
 }
 
 
-# pdftex pdftexconfig.tex format
+# pdftex pdftexconfig.dat format
 # /--- pdftexconfig.tex ---
 # |...
-# |\ifx\pdfpagewidth\undefined\else \pdfpagewidth=210 true mm \fi
-# |\ifx\pagewidth\undefined\else \pagewidth=210 true mm \fi
-# |\ifx\pdfpageheight\undefined\else \pdfpageheight=297 true mm \fi
-# |\ifx\pageheight\undefined\else \pageheight=297 true mm \fi
+# |\pdfpageheight       = 297 true mm
+# |\pdfpagewidth        = 210 true mm
 # |...
 # \------------------------
 #
@@ -322,7 +343,7 @@ sub paper_pdftex {
 
   return($F_ERROR) unless $inp; 
 
-  open(FOO, "<$inp") || die "$0: open($inp) failed: $!";
+  open(FOO, "<$inp") || die "$prg: open($inp) failed: $!";
   my @lines = <FOO>;
   close(FOO);
 
@@ -333,21 +354,21 @@ sub paper_pdftex {
   # read the lines and the last pdfpageswidth/height wins
   for my $idx (0..$#lines) {
     my $l = $lines[$idx];
-    if ($l =~ m/\\(pdf|luatex|)pagewidth\s*=?\s*([0-9.,]+\s*true\s*[^\s]*)/) {
-      if (defined($cpw) && $cpw ne $2) {
+    if ($l =~ m/^\s*\\pdfpagewidth\s*=\s*([0-9.,]+\s*true\s*[^\s]*)/) {
+      if (defined($cpw) && $cpw ne $1) {
         tl_warn("TLPaper: inconsistent paper sizes in $inp for page width! Please fix that.\n");
         return $F_ERROR;
       }
-      $cpw = $2;
+      $cpw = $1;
       push @cpwidx, $idx;
       next;
     }
-    if ($l =~ m/\\(pdf|luatex|)pageheight\s*=?\s*([0-9.,]+\s*true\s*[^\s]*)/) {
-      if (defined($cph) && $cph ne $2) {
+    if ($l =~ m/^\s*\\pdfpageheight\s*=\s*([0-9.,]+\s*true\s*[^\s]*)/) {
+      if (defined($cph) && $cph ne $1) {
         tl_warn("TLPaper: inconsistent paper sizes in $inp for page height! Please fix that.\n");
         return $F_ERROR;
       }
-      $cph = $2;
+      $cph = $1;
       push @cphidx, $idx;
       next;
     }
@@ -377,6 +398,17 @@ sub paper_pdftex {
       for my $p (keys %pdftex_papersize) {
         info("$p\n") unless ($p eq $currentpaper);
       }
+    } elsif ($newpaper eq "--json") {
+      my @ret = ();
+      push @ret, "$currentpaper";
+      for my $p (keys %pdftex_papersize) {
+        push @ret, $p unless ($p eq $currentpaper);
+      }
+      my %foo;
+      $foo{'program'} = "pdftex";
+      $foo{'file'} = $inp;
+      $foo{'options'} = \@ret;
+      return \%foo;
     } elsif ($newpaper eq "--returnlist") {
       my @ret = ();
       push @ret, "$currentpaper";
@@ -399,13 +431,11 @@ sub paper_pdftex {
           for my $idx (@cpwidx) {
             ddebug("TLPaper: before line: $lines[$idx]");
             ddebug("TLPaper: replacement: $newwidth\n");
-            $lines[$idx] =~ s/pagewidth\s*=\s*[0-9.,]+\s*true\s*[^\s]*/pagewidth=$newwidth/;
+            $lines[$idx] =~ s/^\s*\\pdfpagewidth\s*=\s*[0-9.,]+\s*true\s*[^\s]*/\\pdfpagewidth        = $newwidth/;
             ddebug("TLPaper: after line : $lines[$idx]");
           }
         } else {
-          my $addlines = '\ifx\pdfpagewidth\undefined\else \pdfpagewidth=' . $newwidth . ' \fi' . "\n";
-          $addlines .= '\ifx\luatexpagewidth\undefined\else \luatexpagewidth=' . $newwidth . ' \fi' . "\n";
-          $addlines .= '\ifx\pagewidth\undefined\else \pagewidth=' . $newwidth . ' \fi' . "\n";
+          my $addlines = "\\pdfpagewidth        = $newwidth\n";
           if (defined($endinputidx)) {
             $lines[$endinputidx] = $addlines . $lines[$endinputidx];
           } else {
@@ -416,36 +446,34 @@ sub paper_pdftex {
           for my $idx (@cphidx) {
             ddebug("TLPaper: before line: $lines[$idx]");
             ddebug("TLPaper: replacement: $newheight\n");
-            $lines[$idx] =~ s/pageheight\s*=\s*[0-9.,]+\s*true\s*[^\s]*/pageheight=$newheight/;
+            $lines[$idx] =~ s/^\s*\\pdfpageheight\s*=\s*[0-9.,]+\s*true\s*[^\s]*/\\pdfpageheight       = $newheight/;
             ddebug("TLPaper: after line : $lines[$idx]");
           }
         } else {
-          my $addlines = '\ifx\pdfpageheight\undefined\else \pdfpageheight=' . $newheight . ' \fi' . "\n";
-          $addlines .= '\ifx\luatexpageheight\undefined\else \luatexpageheight=' . $newheight . ' \fi' . "\n";
-          $addlines .= '\ifx\pageheight\undefined\else \pageheight=' . $newheight . ' \fi' . "\n";
+          my $addlines = "\\pdfpageheight       = $newheight";
           if (defined($endinputidx)) {
             $lines[$endinputidx] = $addlines . $lines[$endinputidx];
           } else {
             $lines[$#lines] = $addlines;
           }
         }
-        info("$0: setting paper size for pdftex to $newpaper.\n");
+        info("$prg: setting paper size for pdftex to $newpaper: $outfile\n");
         mkdirhier(dirname($outfile));
         # if we create the outfile we have to call mktexlsr
         TeXLive::TLUtils::announce_execute_actions("files-changed")
           unless (-r $outfile);
         if (!open(TMP, ">$outfile")) {
-          tlwarn("$0: Cannot write to $outfile: $!\n");
+          tlwarn("$prg: Cannot write to $outfile: $!\n");
           tlwarn("Not setting paper size for pdftex.\n");
           return($F_ERROR);
         }
         for (@lines) { print TMP; }
-        close(TMP) || warn "$0: close(>$outfile) failed: $!";
+        close(TMP) || warn "$prg: close(>$outfile) failed: $!";
         TeXLive::TLUtils::announce_execute_actions("regenerate-formats");
         # TODO should we return the value of announce_execute action?
         return($F_OK);
       } else {
-        tlwarn("$0: Not a valid paper size for pdftex: $newpaper\n");
+        tlwarn("$prg: Not a valid paper size for pdftex: $newpaper\n");
         return($F_WARNING);
       }
     }
@@ -494,7 +522,7 @@ sub paper_dvips {
 
   return($F_ERROR) unless $inp; 
   
-  open(FOO, "<$inp") || die "$0: open($inp) failed: $!";
+  open(FOO, "<$inp") || die "$prg: open($inp) failed: $!";
   my @lines = <FOO>;
   close(FOO);
 
@@ -527,6 +555,12 @@ sub paper_dvips {
       for my $p (@papersizes) {
         info("$p\n"); # first is already the selected one
       }
+    } elsif ($newpaper eq "--json") {
+      my %foo;
+      $foo{'program'} = "dvips";
+      $foo{'file'} = $inp;
+      $foo{'options'} = \@papersizes;
+      return \%foo;
     } elsif ($newpaper eq "--returnlist") {
       return(\@papersizes);
     } else {
@@ -555,20 +589,20 @@ sub paper_dvips {
           }
           push @newlines, $lines[$idx];
         }
-        info("$0: setting paper size for dvips to $newpaper.\n");
+        info("$prg: setting paper size for dvips to $newpaper: $outfile\n");
         mkdirhier(dirname($outfile));
         # if we create the outfile we have to call mktexlsr
         TeXLive::TLUtils::announce_execute_actions("files-changed")
           unless (-r $outfile);
         if (!open(TMP, ">$outfile")) {
-          tlwarn("$0: Cannot write to $outfile: $!\n");
+          tlwarn("$prg: Cannot write to $outfile: $!\n");
           tlwarn("Not setting paper size for dvips.\n");
           return ($F_ERROR);
         }
         for (@newlines) { print TMP; }
-        close(TMP) || warn "$0: close(>$outfile) failed: $!";
+        close(TMP) || warn "$prg: close(>$outfile) failed: $!";
       } else {
-        tlwarn("$0: Not a valid paper size for dvips: $newpaper\n");
+        tlwarn("$prg: Not a valid paper size for dvips: $newpaper\n");
         return($F_WARNING);
       }
     }
@@ -645,6 +679,120 @@ sub paper_dvipdfmx {
 sub paper_context {
   my $outtree = shift;
   my $newpaper = shift;
+  my ($outcomp, $filecomp) = setup_names("context");
+  my $dftfile = $paper{'context'}{'default_file'};
+  my $outfile = "$outtree/$outcomp/$filecomp";
+  my $inp = &find_paper_file("context", "tex", $filecomp, "cont-sys.rme", $dftfile);
+
+  return($F_ERROR) unless $inp; 
+
+  open(FOO, "<$inp") || die "$prg: open($inp) failed: $!";
+  my @lines = <FOO>;
+  close(FOO);
+
+  my @idx;
+  my $idxlast;
+  my $endinputidx = -1;
+  my $currentpaper;
+  # read the lines and the last setuppapersize before the endinput wins
+  for my $idx (0..$#lines) {
+    my $l = $lines[$idx];
+    if ($l =~ m/^[^%]*\\endinput/) {
+      $endinputidx = $idx;
+      last;
+    }
+    if ($l =~ m/^\s*\\setuppapersize\s*\[([^][]*)\].*$/) {
+      if (defined($currentpaper) && $currentpaper ne $1) {
+        tl_warn("TLPaper: inconsistent paper sizes in $inp! Please fix that.\n");
+        return $F_ERROR;
+      }
+      $currentpaper = $1;
+      $idxlast = $idx;
+      push @idx, $idx;
+      next;
+    }
+  }
+  # if we haven't found a paper line, assume a4
+  $currentpaper || ($currentpaper = "a4");
+  # trying to find the right papersize
+  #
+  if (defined($newpaper)) {
+    if ($newpaper eq "--list") {
+      info("$currentpaper\n");
+      for my $p (keys %pdftex_papersize) {
+        info("$p\n") unless ($p eq $currentpaper);
+      }
+    } elsif ($newpaper eq "--json") {
+      my @ret = ();
+      push @ret, "$currentpaper";
+      for my $p (keys %pdftex_papersize) {
+        push @ret, $p unless ($p eq $currentpaper);
+      }
+      my %foo;
+      $foo{'program'} = "context";
+      $foo{'file'} = $inp;
+      $foo{'options'} = \@ret;
+      return \%foo;
+    } elsif ($newpaper eq "--returnlist") {
+      my @ret = ();
+      push @ret, "$currentpaper";
+      for my $p (keys %pdftex_papersize) {
+        push @ret, $p unless ($p eq $currentpaper);
+      }
+      return \@ret;
+    } else {
+      my $found = 0;
+      for my $p (keys %pdftex_papersize) {
+        if ($p eq $newpaper) {
+          $found = 1;
+          last;
+        }
+      }
+      if ($found) {
+        if (@idx) {
+          for my $idx (@idx) {
+            ddebug("TLPaper: before line: $lines[$idx]");
+            ddebug("TLPaper: replacement: $newpaper\n");
+            $lines[$idx] =~ s/setuppapersize\s*\[([^][]*)\]\[([^][]*)\]/setuppapersize[$newpaper][$newpaper]/;
+            ddebug("TLPaper: after line : $lines[$idx]");
+          }
+        } else {
+          my $addlines = "\\setuppapersize[$newpaper][$newpaper]\n";
+          if (defined($endinputidx)) {
+            $lines[$endinputidx] = $addlines . $lines[$endinputidx];
+          } else {
+            $lines[$#lines] = $addlines;
+          }
+        }
+        info("$prg: setting paper size for context to $newpaper: $outfile\n");
+        mkdirhier(dirname($outfile));
+        # if we create the outfile we have to call mktexlsr
+        TeXLive::TLUtils::announce_execute_actions("files-changed")
+          unless (-r $outfile);
+        if (!open(TMP, ">$outfile")) {
+          tlwarn("$prg: Cannot write to $outfile: $!\n");
+          tlwarn("Not setting paper size for context.\n");
+          return($F_ERROR);
+        }
+        for (@lines) { print TMP; }
+        close(TMP) || warn "$prg: close(>$outfile) failed: $!";
+        TeXLive::TLUtils::announce_execute_actions("regenerate-formats");
+        # TODO should we return the value of announce_execute action?
+        return($F_OK);
+      } else {
+        tlwarn("$prg: Not a valid paper size for context: $newpaper\n");
+        return($F_WARNING);
+      }
+    }
+  } else {
+    info("Current context paper size (from $inp): $currentpaper\n");
+  }
+  return($F_OK);
+}
+
+sub paper_context_old {
+  my $outtree = shift;
+  my $newpaper = shift;
 
   my ($outcomp, $filecomp) = setup_names("context");
   my $dftfile = $paper{'context'}{'default_file'};
@@ -655,11 +803,11 @@ sub paper_context {
   my @sizes = keys %pdftex_papersize;
   # take care here, the \\\\ are necessary in some places and not in 
   # some others because there is no intermediate evaluation
-  return &paper_do_simple($inp, "context", '^\s*\\\\setuppapersize\s*', 
-            '^\s*\\\\setuppapersize\s*\[([^][]*)\].*$',
+  return &paper_do_simple($inp, "context", '^\s*%?\s*\\\\setuppapersize\s*', 
+            '^\s*%?\s*\\\\setuppapersize\s*\[([^][]*)\].*$',
             sub {
               my ($ll,$np) = @_;
-              if ($ll =~ m/^\s*\\setuppapersize\s*/) {
+              if ($ll =~ m/^\s*%?\s*\\setuppapersize\s*/) {
                 return("\\setuppapersize[$np][$np]\n");
               } else {
                 return($ll);
@@ -744,6 +892,17 @@ sub paper_do_simple {
       for my $p (@$paplist) {
         info("$p\n") unless ($p eq $currentpaper);
       }
+    } elsif ($newpaper eq "--json") {
+      my @ret = ();
+      push @ret, "$currentpaper";
+      for my $p (@$paplist) {
+        push @ret, $p unless ($p eq $currentpaper);
+      }
+      my %foo;
+      $foo{'program'} = $prog;
+      $foo{'file'} = $inp;
+      $foo{'options'} = \@ret;
+      return \%foo;
     } elsif ($newpaper eq "--returnlist") {
       my @ret = ();
       push @ret, $currentpaper;
@@ -774,23 +933,23 @@ sub paper_do_simple {
         if (!$foundcfg) {
           push @newlines, &$bl($defaultline, $newpaper);
         }
-        info("$0: setting paper size for $prog to $newpaper.\n");
+        info("$prg: setting paper size for $prog to $newpaper: $outp\n");
         mkdirhier(dirname($outp));
         # if we create the outfile we have to call mktexlsr
         TeXLive::TLUtils::announce_execute_actions("files-changed")
           unless (-r $outp);
         if (!open(TMP, ">$outp")) {
-          tlwarn("$0: Cannot write to $outp: $!\n");
+          tlwarn("$prg: Cannot write to $outp: $!\n");
           tlwarn("Not setting paper size for $prog.\n");
           return ($F_ERROR);
         }
         for (@newlines) { print TMP; }
-        close(TMP) || warn "$0: close(>$outp) failed: $!";
+        close(TMP) || warn "$prg: close(>$outp) failed: $!";
         TeXLive::TLUtils::announce_execute_actions("regenerate-formats")
           if ($prog eq "context");
         return($F_OK);
       } else {
-        tlwarn("$0: Not a valid paper size for $prog: $newpaper\n");
+        tlwarn("$prg: Not a valid paper size for $prog: $newpaper\n");
         return($F_WARNING);
       }
     }
