@@ -33,6 +33,7 @@ $mirrors = {
     },
     'Japan' => {
       'ftp://ftp.kddilabs.jp/CTAN/' => 1,
+      'ftp://ftp.u-aizu.ac.jp/pub/tex/CTAN/' => 1,
       'http://ftp.jaist.ac.jp/pub/CTAN/' => 1,
       'http://ftp.yz.yamagata-u.ac.jp/pub/CTAN/' => 1
     },
@@ -113,6 +114,7 @@ $mirrors = {
     },
     'Portugal' => {
       'ftp://ftp.di.uminho.pt/pub/ctan/' => 1,
+      'ftp://ftp.ist.utl.pt/pub/ctan/' => 1,
       'http://mirrors.up.pt/pub/CTAN/' => 1
     },
     'Romania' => {
@@ -136,7 +138,6 @@ $mirrors = {
       'http://ftp.acc.umu.se/mirror/CTAN/' => 1
     },
     'Switzerland' => {
-      'http://mirror.switch.ch/ftp/mirror/tex/' => 1,
       'http://www.pirbot.com/mirrors/ctan/' => 1
     },
     'UK' => {
